@@ -8,7 +8,7 @@ Proper docstrings for scipy.optimize.minimize et al.
 Usage::
 
     .. scipy-optimize:function:: scipy.optimize.minimize
-       :impl: scipy.optimize.optimize._minimize_nelder_mead
+       :impl: scipy.optimize._optimize._minimize_nelder_mead
        :method: Nelder-Mead
 
 Produces output similar to autodoc, except
@@ -111,11 +111,15 @@ def wrap_mangling_directive(base_directive):
                 else:
                     options.append((opt_name, None))
             set_default('options', dict(options))
-            set_default('method', self.options['method'].strip())
+            if 'method' in self.options and 'method' in args:
+                set_default('method', self.options['method'].strip())
+            elif 'solver' in self.options and 'solver' in args:
+                set_default('solver', self.options['solver'].strip())
 
+            special_args = {'fun', 'x0', 'args', 'tol', 'callback', 'method',
+                            'options', 'solver'}
             for arg in list(args):
-                if arg not in impl_args and arg not in ('fun', 'x0', 'args', 'tol',
-                                                        'callback', 'method', 'options'):
+                if arg not in impl_args and arg not in special_args:
                     remove_arg(arg)
 
             # XXX deprecation that we should fix someday using Signature (?)
@@ -131,10 +135,12 @@ def wrap_mangling_directive(base_directive):
             # Change "Options" to "Other Parameters", run numpydoc, reset
             new_lines = []
             for line in lines:
+                # Remap Options to the "Other Parameters" numpydoc section
+                # along with correct heading length
                 if line.strip() == 'Options':
                     line = "Other Parameters"
-                elif line.strip() == "-"*len('Options'):
-                    line = "-"*len("Other Parameters")
+                    new_lines.extend([line, "-"*len(line)])
+                    continue
                 new_lines.append(line)
             # use impl_name instead of name here to avoid duplicate refs
             mangle_docstrings(env.app, 'function', impl_name,
