@@ -6,39 +6,65 @@
 
 #include "Python.h"
 #include <setjmp.h>
+#include <numpy/npy_math.h>
+#include "Zeros/zeros.h"
+
+/*
+ * Caller entry point functions
+ */
+
+#ifdef PYPY_VERSION
+    /*
+     * As described in http://doc.pypy.org/en/latest/cpython_differences.html#c-api-differences,
+     * "assignment to a PyTupleObject is not supported after the tuple is used internally,
+     * even by another C-API function call."
+     */
+    #define PyArgs(Operation) PyList_##Operation
+#else
+    /* Using a list in CPython raises "TypeError: argument list must be a tuple" */
+    #define PyArgs(Operation) PyTuple_##Operation
+#endif
 
 typedef struct {
-    int funcalls;
-    int iterations;
-    int error_num;
     PyObject *function;
-    PyObject *args;
+    PyObject *xargs;
     jmp_buf env;
 } scipy_zeros_parameters;
 
-/*
- * Storage for the relative precision of doubles. This is computed when the module
- * is initialized.
- */
 
-#include "Zeros/zeros.h"
 
-#define SIGNERR -1
-#define CONVERR -2
-
-static double scipy_zeros_rtol=0;
-
-double
+static double
 scipy_zeros_functions_func(double x, void *params)
 {
     scipy_zeros_parameters *myparams = params;
-    PyObject *args, *f, *retval=NULL;
+    PyObject *args, *xargs, *item, *f, *retval=NULL;
+    Py_ssize_t i, len;
     double val;
 
-    args = myparams->args;
+    xargs = myparams->xargs;
+    /* Need to create a new 'args' tuple on each call in case 'f' is
+       stateful and keeps references to it (e.g. functools.lru_cache) */
+    len = PyTuple_Size(xargs);
+    /* Make room for the double as first argument */
+    args = PyArgs(New)(len + 1);
+    if (args == NULL) {
+        PyErr_SetString(PyExc_RuntimeError, "Failed to allocate arguments");
+        longjmp(myparams->env, 1);
+    }
+    PyArgs(SET_ITEM)(args, 0, Py_BuildValue("d", x));
+    for (i = 0; i < len; i++) {
+        item = PyTuple_GetItem(xargs, i);
+        if (item == NULL) {
+            Py_DECREF(args);
+            longjmp(myparams->env, 1);
+        }
+        Py_INCREF(item);
+        PyArgs(SET_ITEM)(args, i+1, item);
+    }
+
     f = myparams->function;
-    PyTuple_SetItem(args, 0, Py_BuildValue("d",x));
-    retval=PyObject_CallObject(f,args);
+    retval = PyObject_CallObject(f,args);
+    Py_DECREF(args);
     if (retval == NULL) {
         longjmp(myparams->env, 1);
     }
@@ -47,6 +73,7 @@ scipy_zeros_functions_func(double x, void *params)
     return val;
 }
 
+
 /*
  * Helper function that calls a Python function with extended arguments
  */
@@ -54,18 +81,17 @@ scipy_zeros_functions_func(double x, void *params)
 static PyObject *
 call_solver(solver_type solver, PyObject *self, PyObject *args)
 {
-    double a,b,xtol,zero;
-    int iter,i, len, fulloutput, disp=1, flag=0;
+    double a, b, xtol, rtol, zero;
+    int iter, fulloutput, disp=1, flag=0;
     scipy_zeros_parameters params;
-    jmp_buf env;
-    PyObject *f, *xargs, *item, *fargs=NULL;
+    scipy_zeros_info solver_stats;
+    PyObject *f, *xargs;
 
-    if (!PyArg_ParseTuple(args, "OdddiOi|i",
-                &f, &a, &b, &xtol, &iter, &xargs, &fulloutput, &disp))
-        {
-            PyErr_SetString(PyExc_RuntimeError, "Unable to parse arguments");
-            return NULL;
-        }
+    if (!PyArg_ParseTuple(args, "OddddiOi|i",
+                &f, &a, &b, &xtol, &rtol, &iter, &xargs, &fulloutput, &disp)) {
+        PyErr_SetString(PyExc_RuntimeError, "Unable to parse arguments");
+        return NULL;
+    }
     if (xtol < 0) {
         PyErr_SetString(PyExc_ValueError, "xtol must be >= 0");
         return NULL;
@@ -75,65 +101,46 @@ call_solver(solver_type solver, PyObject *self, PyObject *args)
         return NULL;
     }
 
-    len = PyTuple_Size(xargs);
-    /* Make room for the double as first argument */
-    fargs = PyTuple_New(len + 1);
-    if (fargs == NULL) {
-        PyErr_SetString(PyExc_RuntimeError,
-                "Failed to allocate argument tuple");
+    params.function = f;
+    params.xargs = xargs;
+
+    if (!setjmp(params.env)) {
+        /* direct return */
+        solver_stats.error_num = 0;
+        zero = solver(scipy_zeros_functions_func, a, b, xtol, rtol,
+                      iter, (void*)&params, &solver_stats);
+    } else {
+        /* error return from Python function */
         return NULL;
     }
 
-    for (i = 0; i < len; i++) {
-        item = PyTuple_GetItem(xargs, i);
-        if (item == NULL) {
-            Py_DECREF(fargs);
+    if (solver_stats.error_num != CONVERGED) {
+        if (solver_stats.error_num == SIGNERR) {
+            PyErr_SetString(PyExc_ValueError,
+                    "f(a) and f(b) must have different signs");
             return NULL;
         }
-        Py_INCREF(item);
-        PyTuple_SET_ITEM(fargs, i+1, item);
-    }
-
-    params.function = f;
-    params.args = fargs;
-
-    if (!setjmp(env)) {
-        /* direct return */
-        memcpy(params.env, env, sizeof(jmp_buf));
-        params.error_num = 0;
-        zero = solver(scipy_zeros_functions_func, a, b,
-                xtol, scipy_zeros_rtol, iter, (default_parameters*)&params);
-        Py_DECREF(fargs);
-        if (params.error_num != 0) {
-            if (params.error_num == SIGNERR) {
-                PyErr_SetString(PyExc_ValueError,
-                        "f(a) and f(b) must have different signs");
+        if (solver_stats.error_num == CONVERR) {
+            if (disp) {
+                char msg[100];
+                PyOS_snprintf(msg, sizeof(msg),
+                        "Failed to converge after %d iterations.",
+                        solver_stats.iterations);
+                PyErr_SetString(PyExc_RuntimeError, msg);
                 return NULL;
             }
-            if (params.error_num == CONVERR) {
-                if (disp) {
-                    char msg[100];
-                    PyOS_snprintf(msg, sizeof(msg),
-                            "Failed to converge after %d iterations.",
-                            params.iterations);
-                    PyErr_SetString(PyExc_RuntimeError, msg);
-                    flag = 1;
-                    return NULL;
-                }
-            }
-        }
-        if (fulloutput) {
-            return Py_BuildValue("diii",
-                    zero, params.funcalls, params.iterations, flag);
-        }
-        else {
-            return Py_BuildValue("d", zero);
+            flag = CONVERR;
         }
     }
     else {
-        /* error return from Python function */
-        Py_DECREF(fargs);
-        return NULL;
+        flag = CONVERGED;
+    }
+    if (fulloutput) {
+        return Py_BuildValue("diii",
+                zero, solver_stats.funcalls, solver_stats.iterations, flag);
+    }
+    else {
+        return Py_BuildValue("d", zero);
     }
 }
 
@@ -166,7 +173,7 @@ _brentq(PyObject *self, PyObject *args)
 }
 
 /*
- * Standard Python module inteface
+ * Standard Python module interface
  */
 
 static PyMethodDef
@@ -178,16 +185,6 @@ Zerosmethods[] = {
 	{NULL, NULL}
 };
 
-static double __compute_relative_precision()
-{
-    double tol;
-
-    /* Determine relative precision of doubles, assumes binary */
-    for(tol = 1; tol + 1 != 1; tol /= 2);
-    return 2*tol;
-}
-
-#if PY_VERSION_HEX >= 0x03000000
 static struct PyModuleDef moduledef = {
     PyModuleDef_HEAD_INIT,
     "_zeros",
@@ -200,20 +197,12 @@ static struct PyModuleDef moduledef = {
     NULL
 };
 
-PyObject *PyInit__zeros(void)
+PyMODINIT_FUNC
+PyInit__zeros(void)
 {
-    PyObject *m, *d, *s;
+    PyObject *m;
 
     m = PyModule_Create(&moduledef);
 
-    scipy_zeros_rtol = __compute_relative_precision();
-
     return m;
 }
-#else
-PyMODINIT_FUNC init_zeros(void)
-{
-        Py_InitModule("_zeros", Zerosmethods);
-        scipy_zeros_rtol = __compute_relative_precision();
-}
-#endif
